#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <stdio.h>
#include <math.h>
#include "GraphP.h"

/* this procedure translates from world cordinates to 
   viewspace? coordinates (still in 3d)   
*/

void translate(wid,x,y,z,nx,ny,nz)
	GraphWidget   wid;
	float         x, y, z;
	float         *nx,*ny,*nz;

	{
	  *nx = x * wid->graph.project[0][0] +
	       y * wid->graph.project[1][0] +
	       z * wid->graph.project[2][0] +
	       wid->graph.project[3][0];

	  *ny = x * wid->graph.project[0][1] +
	       y * wid->graph.project[1][1] +
	       z * wid->graph.project[2][1] +
	       wid->graph.project[3][1];

	  *nz = x * wid->graph.project[0][2] +
	       y * wid->graph.project[1][2] +
	       z * wid->graph.project[2][2] +
	       wid->graph.project[3][2];

	 }

void PrepareProject(wid,olx,oly,oux,ouy)
	GraphWidget  wid;
	float        *olx,*oly,*oux,*ouy;

	{

	   *olx = wid->graph.graphXMin;
	   *oly = wid->graph.graphYMin;
	   *oux = wid->graph.graphXMax;
	   *ouy = wid->graph.graphYMax;

	   wid->graph.graphXMin = -wid->graph.thd.d2/2;
	   wid->graph.graphYMin = -wid->graph.thd.d3/2;
	   wid->graph.graphXMax = wid->graph.thd.d2/2;
	   wid->graph.graphYMax = wid->graph.thd.d3/2;
       }

void Project(wid,x,y,z,nx,ny)
	GraphWidget   wid; 
        float         x,y,z,*nx,*ny;
	{
	   float tx;

	   tx = z/wid->graph.thd.d1+1;
	   if (tx != 0) {
	        *nx =  x  / (tx);
	        *ny =  y / (tx);
	   } else {
	      *nx = x*wid->graph.graphXMax; 
	      *ny = y*wid->graph.graphYMax;
	   }

	}

void StopProject (wid,olx,oly,oux,ouy)
	GraphWidget   wid;
	float         olx,oly,oux,ouy;

	{
	   wid->graph.graphXMin = olx;
	   wid->graph.graphYMin = oly;
	   wid->graph.graphXMax = oux;
	   wid->graph.graphYMax = ouy;
	}

void XGPDrawLine3D(wid, x1,y1,z1,a1,b1,c1,tflag)
	GraphWidget   wid;
	float         x1,y1,z1,a1,b1,c1;
	int           tflag;

/* tflag added 2/25/90 if true translation needs to be done */

	{
	   float   nx,ny,nnx,nny;
	   float      olx,oly,oux,ouy;
	   float   x,y,z,a,b,c;

           if (tflag == TRUE ) {
	       translate(wid,x1,y1,z1,&x,&y,&z);
	       translate(wid,a1,b1,c1,&a,&b,&c);
	   } else {
	       x = x1; y = y1; z = z1;
	       a = a1; b = b1; c = c1;
	   }
           PrepareProject(wid,&olx,&oly,&oux,&ouy);

	   Project(wid,x,y,z,&nx,&ny);
	   Project(wid,a,b,c,&nnx,&nny);

	   drawline(wid,nx,ny,nnx,nny);

	   StopProject(wid,olx,oly,oux,ouy);
	   }

void XGPDraw3DPoint(wid,x,y,z)
	GraphWidget   wid;
	float         x,y,z;
	{
	   XGPDrawLine3D(wid,x,y,z,x,y,z,TRUE);
	}

#define degrees_to_radians 0.017453293
#define epsalon     0.000005

void resetmatrix(wid)
       GraphWidget  wid;

       /* this procedure is responcible for making the rotation matrix */

       {
	 float  Fx,Fy,Fz,ca,ct,cf,sa,st,sf;
         float  d1,d2,d3;
         float  xp,zp,yp,zpp;
	 float  x,y,z;

         xp = wid->graph.thd.eye.x-wid->graph.thd.center.x;
         yp = wid->graph.thd.eye.y-wid->graph.thd.center.y;
	 zp = wid->graph.thd.eye.z-wid->graph.thd.center.z;

    
	 Fx = wid->graph.thd.center.x;
	 Fy = wid->graph.thd.center.y;
	 Fz = wid->graph.thd.center.z;

	 d2 = sqrt(xp*xp+yp*yp+zp*zp);
	 if (d2 < 0) {
	     d1 = 0.5;
	 }  else {
	     d1 = -0.5;
	 }

	 x = wid->graph.thd.center.x+xp*d1;
	 y = wid->graph.thd.center.y+yp*d1;
	 z = wid->graph.thd.center.z+zp*d1;

	 xp = -(wid->graph.thd.center.x -x);
	 yp = -(wid->graph.thd.center.y -y);
	 zp = -(wid->graph.thd.center.z -z);


	 d1 = sqrt(xp*xp+zp*zp);
	 if(d1 < epsalon){
	        ct = 1;
	        st = 0;
	 } else {
	        ct = -zp/d1;
	        st = xp/d1;
	 }

	 zpp = (-(xp*xp)-(zp*zp))/d1 ;
	 d2 = sqrt(xp*xp+yp*yp+zp*zp);
         if(d2 < epsalon) {	
	       cf = 1;
	       sf = 0;
	 } else {
	       cf = -zpp/d2;
	       sf = -yp/d2;
	 }

	 if (d1 < epsalon) {
	       cf = 0;
	 }
	 ca = cos(wid->graph.thd.rot*degrees_to_radians);
	 sa = sin(wid->graph.thd.rot*degrees_to_radians);

 wid->graph.project[0][0] = ct*ca-st*sf*sa; 
 wid->graph.project[0][1] = ct*sa+st*sf*ca; 
 wid->graph.project[0][2] = st*cf;
 wid->graph.project[0][3] = 0;
 wid->graph.project[1][0] = -cf*sa; 
 wid->graph.project[1][1] = cf*ca;
 wid->graph.project[1][2] = -sf;
 wid->graph.project[1][3] = 0;
 wid->graph.project[2][0] = st*ca+ct*sf*sa;
 wid->graph.project[2][1] = st*sa-ct*sf*ca;
 wid->graph.project[2][2] = -ct*cf;
 wid->graph.project[2][3] = 0;
 wid->graph.project[3][0] = (-Fx*ct-Fz*st)*ca+Fy*cf*sa+(Fx*st-Fz*ct)*sf*sa;
 wid->graph.project[3][1] = (-Fx*ct-Fz*st)*sa-Fy*cf*ca-(Fx*st-Fz*ct)*sf*ca;
 wid->graph.project[3][2] = Fy*sf-(Fx*st-Fz*ct)*cf;
 wid->graph.project[3][3] = 1;
	}

void XGPSet3DViewing(wid,a,b,c,x,y,z,d1,d2,d3,r)
       GraphWidget  wid;
       float a,b,c,x,y,z,d1,d2,d3,r;

       {
	  float   tx,ty,tz;

       wid->graph.thd.eye.x = a;
       wid->graph.thd.eye.y = b;
       wid->graph.thd.eye.z = c;
       wid->graph.thd.center.x = x;
       wid->graph.thd.center.y = y;
       wid->graph.thd.center.z = z;
       wid->graph.thd.d1 = d1;
       wid->graph.thd.d2 = d2;
       wid->graph.thd.d3 = d3;
       wid->graph.thd.rot = r;

       resetmatrix(wid);
    }
