#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include "GraphP.h"
#include "pars.h"
#include "y.tab.h"
#include <math.h>

extern void XGPDrawLine();
extern void XGPDrawPoint();

/*
 ***********************************************************************
 *
 * File:         x11graph3d.c
 * Description:  graphics routines for 3D graphing
 * Purpose:      to draw 3D surface from data points in file
 * Author:       Paul S. Wang
 * Created:      7/20/88
 *
 * (c) Copyright 1988, Kent State University all rights reserved 
 *
 ***********************************************************************
 */

/* displaying polygons with possible hidden line removal */

void FreeList(p)
	POLY  *p;
	{
	   if (p != NULL) {
	      FreeList(p->next);
              free(p);
	   }
	}
	

#define EPSALON 0.00001

int hlhstest4(p,q)
	POLY  *p,*q;
	{
	   double  a,b,c,d;
           double  dist;

	   dist = sqrt((double)(p->A*p->A+p->B*p->B+p->C*p->C));
	   a = (q->p1.x*p->A + q->p1.y*p->B+ q->p1.z*p->C+p->D)/dist;
	   b = (q->p2.x*p->A + q->p2.y*p->B+ q->p2.z*p->C+p->D)/dist;
	   c = (q->p3.x*p->A + q->p3.y*p->B+ q->p3.z*p->C+p->D)/dist;
	   d = (q->p4.x*p->A + q->p4.y*p->B+ q->p4.z*p->C+p->D)/dist;
	   if (fabs(a) < EPSALON) { a = 0;}
	   if (fabs(b) < EPSALON) { b = 0;}
	   if (fabs(c) < EPSALON) { c = 0;}
	   if (fabs(d) < EPSALON) { d = 0;}
	   if (p->EyeDistance > 0) {
	       if ((a>=0)&&(b>=0)&&(c>=0)&&(d>=0)) {
	           return(True);
	       } else {
	           return(False);
	       }
	   } else if (p->EyeDistance < 0) {
	       if ((a<=0)&&(b<=0)&&(c<=0)&&(d<=0)) {
	           return(True);
	       } else {
		   return(False);
	       }
	   } else {
	       return(True);
	   }
	}

int hlhstest3(p,q)
	POLY  *p,*q;
	{
	   double  a,b,c,d;
           double  dist;

	   dist = sqrt((double)(q->A*q->A+q->B*q->B+q->C*q->C));
	   a = (p->p1.x*q->A + p->p1.y*q->B+ p->p1.z*q->C+q->D)/dist;
	   b = (p->p2.x*q->A + p->p2.y*q->B+ p->p2.z*q->C+q->D)/dist;
	   c = (p->p3.x*q->A + p->p3.y*q->B+ p->p3.z*q->C+q->D)/dist;
	   d = (p->p4.x*q->A + p->p4.y*q->B+ p->p4.z*q->C+q->D)/dist;
	   if (fabs(a) < EPSALON) { a = 0;}
	   if (fabs(b) < EPSALON) { b = 0;}
	   if (fabs(c) < EPSALON) { c = 0;}
	   if (fabs(d) < EPSALON) { d = 0;}
	   if (q->EyeDistance < 0) {
	       if ((a>=0)&&(b>=0)&&(c>=0)&&(d>=0)) {
	           return(True);
	       } else {
	           return(False);
	       }
	   } else if (q->EyeDistance > 0) {
	       if ((a<=0)&&(b<=0)&&(c<=0)&&(d<=0)) {
	           return(True);
	       } else {
		   return(False);
	       }
	   } else {
	       return(True);
	   }
	}

int test(p,q)
	POLY *p,*q;
	{
	    if ((p->xmax < q->xmin) || (p->xmin > q->xmax)) {
		return(True);
	    } else if ((p->ymax < q->ymin) || (p->ymin > q->ymax)) {
		return(True);
	    } else if (hlhstest3(p,q)) {
		return(True);
	    } else if (hlhstest4(p,q)) {
		return(True);
	    } else {
		return(False);
	    }
        }

void Resolve(p)
	POLY *p;
	{
	POLY  *max,*now,temp,*foo,*bar;

	/* find all the polygons we need to compare against*/
	
	max = p->next;
	while ((max != NULL) && (p->zmin < max->zmax)){
	      max = max->next;
	}

	/* for each polygon in the compare list*/
	now = p->next;
	while(now != max) {
	    if (test(p,now) == False){
	      if (now->Swapped == 0) {
	           /* mark as swapped */
			
	           p->Swapped = 1;

	           /* swap p and now */

		   foo = p->next;
		   bar = now->next;
	           bcopy(p,&temp,sizeof(POLY));
	           bcopy(now,p,sizeof(POLY));
	           bcopy(&temp,now,sizeof(POLY));
		   p->next = foo;
		   now->next = bar;
		   now = p->next; 
	      } else {
		  printf("clipping \n");
		  /*they have already been swapped.  Must be clipped */
		  now = now->next;
	      }
	    } else {
		now = now->next;
	    }
        }   /* while */
	for(now=p->next;now != max;now= now->next){
	     now->Swapped = 0;
	}
	}

void drawlist(wid)
	GraphWidget  wid;
	{
	   POLY   *list;
           float   x1,y1,x2,y2,x3,y3,x4,y4;
           float      olx,oly,oux,ouy;
           float   x,y,z,a,b,c;
           XPoint   p[4];
           Display  *disp;
	   int      flag;

           PrepareProject(wid,&olx,&oly,&oux,&ouy);
	   list = wid->graph.polylist;

	   while (list != NULL) {
	       Resolve(list);
               Project(wid,list->p1.x,list->p1.y,list->p1.z,&x1,&y1);
               Project(wid,list->p2.x,list->p2.y,list->p2.z,&x2,&y2);
               Project(wid,list->p3.x,list->p3.y,list->p3.z,&x3,&y3);
               Project(wid,list->p4.x,list->p4.y,list->p4.z,&x4,&y4);
	       flag =(ChangeColor(wid,list->parent)||
		      ChangeStyle(wid,list->parent));
               DrawPolygon(wid,x1,y1,x2,y2,x3,y3,x4,y4);
	       if (flag != 0 ) {
	          Change_Line_Style (wid,0,TSOLID);
	          XSetForeground(XtDisplay(wid),wid->graph.XGPgc,
			  wid->graph.foreground);
               }
               list = list->next;
           }	       
           StopProject(wid,olx,oly,oux,ouy);
           FreeList(wid->graph.polylist);
           wid->graph.polylist = NULL;
	}
	
void addtolist(p1,p2)
	POLY   *p1,*p2;
	{
	     if (p1->next == NULL) {
		p1->next = p2;
		p2->next = NULL;
	     } else if (p1->next->zmin < p2->zmin) {
		p2 -> next = p1->next;
		p1 -> next = p2;
	     } else {
		addtolist(p1->next,p2);
	     }
	}

float max(a,b,c,d)
        float	 a,b,c,d;
        {
	    float small;

	    small = a;
	    if (small < b) { small = b;}
	    if (small < c) { small = c;}
	    if (small < d) { small = d;}
	    return(small);
        }

float min(a,b,c,d)
        float a,b,c,d;
        {
	    float small;

	    small = a;
	    if (small > b) { small = b;}
	    if (small > c) { small = c;}
	    if (small > d) { small = d;}
	    return(small);
        }

int testfit(p)
	POLY  *p;
	{
	   double  a,b,c,d;
           double  dist;

	   dist = sqrt((double)(p->A*p->A+p->B*p->B+p->C*p->C));
	   a = (p->p1.x*p->A + p->p1.y*p->B+ p->p1.z*p->C+p->D)/dist;
	   b = (p->p2.x*p->A + p->p2.y*p->B+ p->p2.z*p->C+p->D)/dist;
	   c = (p->p3.x*p->A + p->p3.y*p->B+ p->p3.z*p->C+p->D)/dist;
	   d = (p->p4.x*p->A + p->p4.y*p->B+ p->p4.z*p->C+p->D)/dist;
	   printf(" %f  %f  %f  %f  \n",a,b,c,d);
	}


void calcPlaneEq(temp)
     POLY  *temp;
     {
        temp->A = temp->p1.y*(temp->p2.z-temp->p3.z)+
                  temp->p2.y*(temp->p3.z-temp->p1.z)+
                  temp->p3.y*(temp->p1.z-temp->p2.z);
        temp->B = -(temp->p1.x*(temp->p2.z-temp->p3.z)+
                  temp->p2.x*(temp->p3.z-temp->p1.z)+
                  temp->p3.x*(temp->p1.z-temp->p2.z));
        temp->C = temp->p1.x*(temp->p2.y-temp->p3.y)+
                  temp->p2.x*(temp->p3.y-temp->p1.y)+
                  temp->p3.x*(temp->p1.y-temp->p2.y);
        temp->D = -(temp->p1.x*(temp->p2.y*temp->p3.z-temp->p3.y*temp->p2.z)+
                  temp->p2.x*(temp->p3.y*temp->p1.z-temp->p1.y*temp->p3.z)+
                  temp->p3.x*(temp->p1.y*temp->p2.z-temp->p2.y*temp->p1.z));
     }

void AddPolygon(wid,obj,p1,p2,p3,p4)
	GraphWidget        wid;
	XGPObject         *obj;
	XtXGPPointStruct   p1,p2,p3,p4;
	{
         POLY   *temp;
         XtXGPPointStruct  foo;
         double  dist;
	   
	 temp = (POLY*) XGPmalloc(wid,sizeof(POLY));
	 temp->parent = obj;
	 translate(wid,p1.x,p1.y,p1.z,&foo.x,&foo.y,&foo.z);
	 temp->p1 = foo;
	 translate(wid,p2.x,p2.y,p2.z,&foo.x,&foo.y,&foo.z);
	 temp->p2 = foo;
	 translate(wid,p3.x,p3.y,p3.z,&foo.x,&foo.y,&foo.z);
	 temp->p3 = foo;
	 translate(wid,p4.x,p4.y,p4.z,&foo.x,&foo.y,&foo.z);
	 temp->p4 = foo;
         temp->zmax = max(temp->p1.z,temp->p2.z,temp->p3.z,temp->p4.z);
         temp->zmin = min(temp->p1.z,temp->p2.z,temp->p3.z,temp->p4.z);
         temp->xmax = max(temp->p1.x,temp->p2.x,temp->p3.x,temp->p4.x);
         temp->xmin = min(temp->p1.x,temp->p2.x,temp->p3.x,temp->p4.x);
         temp->ymax = max(temp->p1.y,temp->p2.y,temp->p3.y,temp->p4.y);
         temp->ymin = min(temp->p1.y,temp->p2.y,temp->p3.y,temp->p4.y);
	 temp->Swapped = 0;
	 calcPlaneEq(temp); 
	 dist = sqrt((double)(temp->A*temp->A+temp->B*temp->B+temp->C*temp->C));
	 temp->EyeDistance = (temp->A*wid->graph.thd.eye.x+
			      temp->B*wid->graph.thd.eye.y+
			      temp->C*wid->graph.thd.eye.z+temp->D)/dist;
	 if (NULL == wid->graph.polylist) {
	     temp->next =  NULL;
	     wid->graph.polylist = temp;
	 } else if (temp->zmin > wid->graph.polylist->zmin) {
	     temp->next =  wid->graph.polylist;
	     wid->graph.polylist = temp;
	 } else {
	    addtolist(wid->graph.polylist,temp);
	}
    }
