#include <X11/copyright.h>

/* $XConsortium: GraphP.h,v 1.2 88/10/25 17:37:59 swick Exp $ */
/* Copyright	Massachusetts Institute of Technology	1987, 1988 */

#ifndef _GraphP_h
#define _GraphP_h

/* include superclass private header file */
#include <X11/IntrinsicP.h>
#include <X11/CoreP.h>

#include "Graph.h"
#include "ObjectsP.h"

#include <stdio.h>

/* define unique representation types not found in <X11/StringDefs.h> */

#define XtRGraphResource		"GraphResource"

#ifndef TRUE
#define TRUE        1
#endif
#ifndef FALSE 
#define FALSE       0
#endif
#define OutOfMemmory   1

/* the following will be used in the routines in GraphDeref and the ones
   that call these routine                                                */

#define displayit   0
#define codeit      1

/* the folowing are for the hard copy routines */

#define cm_to_inch 0.3837008
#define ESC (char) 27  /* ascii escape for the laserjet and paintjet */

typedef struct {
                int     Units,   /* inches or cm */
                        dpi,     /* dots per inch on output */
			depth,   /* the depth of output picture */
                        otype;   /* type of output  (tex xgp X ) */
                float   height,  /* height of output picture */
                        width;   /* width of output picture */
                char    *path;   /* the name of the output file */
        } OutputParam;


typedef struct {
    float  Xscale,   /* the x scale of the axis   (space between marks) */
	   Yscale,   /* the y scale of the axis   (space between marks) */
	   TickXmax,     /* the high point on the x axis                    */
	   TickYmax,     /* the high point on the y axis                    */
	   TickXmin,     /* the low tickmark on the x axis                  */
	   TickYmin,     /* the low tickmark on the y axis                  */
           LabXmax,     /* the high point on the x axis                    */
           LabYmax,     /* the high point on the y axis                    */
           LabXmin,     /* the low tickmark on the x axis                  */
           LabYmin,     /* the low tickmark on the y axis                  */
	   XCenter,      /* the center of the axis                      */
	   YCenter;      /* the center of the axis                      */
    int    Visible,  /* if true axis are visible otherwise no axis      */
	   AutoAdjust,    /* automatically adjust stuff                 */
           XStyle,    /*   the line style for the x axis                */
           YStyle,    /*   the line style for the y axis                */
	   XWidth,   /* the width of the x axis                         */
	   YWidth,   /* the width of the y axis  (in pixels             */
           XLabelVisible, /* if the labels are visible                  */
	   YLabelVisible, /* if the y lablels are visible               */
	   XTickVisible, /* if the tickmarks are visible                */
           YTickVisible, /* are the y ticmarks visible?                 */
	   Xjustify, /* where to put the lables                         */
	   Yjustify, /* where to put the Y lables                       */
	   XIntPre,  /* the whole number precision on the x axis        */
	   YIntPre,  /* the whole number precision on the y axis        */
	   XDecPre,  /* the decimal precision on the x axis             */
	   YDecPre,  /* the decimal precision on the y axis             */
	   XLabelSkip, /* Label only every I tickmark on the X axis     */
	   YLabelSkip, /* Label only every i tickmark on the Y axis     */
	   XLabelFont, /* the font for the x axis                       */
	   YLabelFont; /* the font for the y axis                       */
    char   *LabelColor,
	   *AxisColor,
	   *TickColor;
}ReferenceStuff;

typedef struct {
	float   start,     /* the start of the line */
		stop,      /* the end of the line */
		otherv1,   /* an other coord value */
		otherv2,   /* the 3rd coord value  */
		center,    /* the center of the line (where tickmarks start)*/
		tickdist;  /* the distance between tickmarks */
	int     visibility, /* if the line is visible */
		tickvis;    /* tickmark visibility    */
} D3reflinedef;

typedef struct {
        float 	xmin,
		xmax,
		ymin,
		ymax,
		zmin,
		zmax;
        int     visibility;
	char    *AxisColor;
}D3Boxdef;

typedef struct {
	float      xmax,xmin,ymax,ymin,zmax,zmin;
	float      dxmin,dxmax,dymin,dymax;
	int        valid,dvalid;
}WorldMinMax;

typedef struct {
  	D3reflinedef   xline,
		      yline,
		      zline;
        char   *LabelColor,
    	       *AxisColor,
    	       *TickColor;
} d3axisdef;

#define CENTER 0
#define UP      1
#define RIGHT   2
#define DOWN    3
#define LEFT    4
#define HOROZONTAL   0
#define VERTICAL     1
/* the following will be used for working with a file  */

typedef struct {
	  FILE   *outfile1;
	  int    outflag;
}FileStuff;

typedef struct {
    int    empty;
} GraphClassPart;

typedef struct _GraphClassRec {
    CoreClassPart	core_class;
    GraphClassPart	graph_class;
} GraphClassRec;

typedef struct {
	XtXGPPointStruct   eye,center;
	float d1,d2,d3, 
	      rot;
}threedposition;

extern GraphClassRec graphClassRec;

#define Def        -1
#define Default    -1
#define Non        -2
#define defaultdim -1 
#define Internal    1

#define MaxAxis     2

typedef struct {
    /*resources */
    int empty;
    XFontStruct *normalfont,     /* the regular font (also font 1) */
		*italicfont,  /* the italic font (also font 2)  */
		*boldfont,    /* the bold font   (also font 3)  */
		*greekfont;   /* the greek font  (also font 4)  */
    Pixel  foreground,
	   background;
    XtCallbackList buisy,response,error;
    /*private State*/
    XGPNameList  *names;
    htblentry *hashtbl[101];
    char     buf[256];
    GC       XGPgc;
    float	     graphXMax,
		     graphYMax,
		     graphXMin,
		     graphYMin;
    CommonAttribute  twoDtrans;
    threedposition   thd;
    float            trans3D[4][4],
		     project[4][4];
    float            twoDxmin, twoDxmax, twoDymin, twoDymax;
    int              twoDvaluesFlag,Update2Dminmax;
    OutputParam      *HardCopyStuff;
    ReferenceStuff   axis[MaxAxis],
                     grid,
                     horilin,
                     virtlin;
    d3axisdef        axis3d;
    D3Boxdef         viewbox;
    FileStuff        files;
    int              mode; /* 2d 3d 2donly 3donly defaultdim (2d) */
    int              modeadj;   /* true change when 3d object comes in to 3d */
    char             *StringHold; /* for buildcommand */
    int              ParenCount,StringLength; /* for buildcommand */
    int              ColormapMode,fghold,bghold,SUImode,Redraw;
    int              Hidden;
    POLY             *polylist;
    WorldMinMax      extremes;
} GraphPart;


typedef struct _GraphRec {
    CorePart		core;
    GraphPart    	graph;
} GraphRec;

#endif  _GraphP_h
