#ifndef _Objects_P_h
#define _Objects_P_h 1

typedef struct {
    float x,y,z;
    } XtXGPPointStruct;

typedef struct _POLY{
    XtXGPPointStruct   p1,p2,p3,p4;
    float              zmax,zmin;
    float              xmax,ymax,xmin,ymin;
    float              A,B,C,D;   /* the plane coefficients */
    float              EyeDistance;
    int                Swapped;
    struct _POLY      *next;
    struct _XGPObject *parent;
}POLY;

/***********************************************************************/
/* The following are declaration of the structures to be used in the   */
/* object manager.  I hope that they are extendable enough to handle   */
/* whatever it is you are trying to add to it.                         */
/***********************************************************************/

typedef struct _XGPNameList{
       struct _XGPObject   *object; 
       struct _XGPNameList *next;
} XGPNameList;

/***********************************************************************/
/* This is a structure which will hold values for the function type    */
/* object.                                                             */
/***********************************************************************/

typedef struct {
	char   *fun;       /* a tree containing the function */
	} XGPFunctionDat;

/***********************************************************************/
/* This is the structure to hold all the points for a complex object   */
/* at this point I plan to make a linked list that contains 100 points */
/* in each node.                                                       */
/***********************************************************************/



typedef struct _PointList {
	 XtXGPPointStruct        point;
	 struct _PointList       *fp, *rp, *op;
	 } PointList;



typedef struct {
	int          number;
	PointList    *points;
	} XGPPointsDat;

/***********************************************************************/
/* the following are different types of pictures you can create with   */
/* a set of data.                                                      */
/***********************************************************************/

typedef int XGPPointsType;

#define ScatterPlot   0   /* individually plotted points  */
#define ConLine       1   /* a line through space         */
	
typedef struct _XGPPointsData {
	XGPPointsType    type;
	struct _XGPPointsData    *data;
	} XGPPointsData;

/***********************************************************************/
/* The following are the structures to hold the primitive data objects */
/***********************************************************************/

typedef struct {
	XtXGPPointStruct  start_point,
			end_point;
	} XGPLineData;

typedef struct {
	XtXGPPointStruct  point;
	} XGPPointData;

typedef struct {
	float radius1,radius2;
	XtXGPPointStruct center;
	} XGPElipseData;

typedef struct _Numlist{
	float           x,y,z;
	struct _Numlist *next;
	}Numlist;

typedef struct {
	Numlist   *points;
	}XGPPolygonData;

typedef struct {
	Numlist  *points;
	}XGPPolylineData; 

typedef struct {
	char     *text;
        XtXGPPointStruct point;	
	}XGPTextData;

typedef struct  _SquareSurfaceStruct{
	float    x,y,z;
	struct _SquareSurfaceStruct    *fp,*sp;
}SquareSurfaceStruct;

typedef struct {
	int   rows;
	SquareSurfaceStruct *data;
}XGPSurfaceData;

typedef union {
	XGPPointData       p_data;
	XGPLineData        l_data;
	XGPElipseData      c_data;
	XGPPolygonData     y_data;
	XGPPolylineData    pl_data;
	XGPTextData        t_data;
	XGPSurfaceData     s_data;
	} XGPPrimitiveData;

typedef int XGPPrimitiveType;

#define Point     0
#define Line      1
#define Elipse    2
#define Polygon   3
#define Polyline  4
#define Text      5
#define Surface   6

typedef struct {
	XGPPrimitiveType     type;
	XGPPrimitiveData     data;
	} XGPPrimitiveDat;

/***********************************************************************/
/* the following are the structures to hold the complex objects        */
/* a complex object is a collecton of simple objects                   */
/***********************************************************************/

typedef struct _ChildList{
	struct _XGPObject *self;
	/* struct _XGPObject *parent; */
	struct _ChildList *siblings;
	} ChildList;

typedef struct {
	ChildList   *children;
	} XGPComplexDat;

/***********************************************************************/
/* this is the toplevel structure for all of the graph objects         */
/***********************************************************************/

typedef union {
	XGPPrimitiveDat p_data;
	XGPComplexDat   c_data;
	} XGPObjectData;

typedef int XGPObjectType;

#define XGPPrimitive    0
#define XGPComplex      3

typedef struct {
       float centerx,
	     centery,
	     centerz,
	     factor1,
	     factor2,
             factor3;
       }transform;

typedef struct {
       int         Hidden;    /* the object is visible or hidden */
       float       transformation[4][4];
       int         linewidth,   /* the width of a line  0 = 1 pixel (Blame X)*/
		   linestyle;   /* a line style see below                    */
       int         fontstyle;   /* the font to use see GraphP.h              */
       int         dimension;  /* a 2D or 3D object   */
       char        *fgcolor;   /* the name of the forground color */
       } CommonAttribute;

typedef struct _XGPObject {
       XGPObjectType     type;
       XGPObjectData     data;
       struct _XGPObject *next, *parent;
       struct _stblentry *aname;
       CommonAttribute   Other;
       } XGPObject;

/*********************************************************************/
/* The shell for this file and for all of the parser scanner was     */
/* was provided by Sanjiva Weerawarana and was origionally developed */
/* for GENCRAY  his Masters Thesis                                          */
/*********************************************************************/

typedef struct _stblentry{ 
        char      *name;                /* actual string    */
        int       length;               /* length of string */
        int       tokencode;            /* see y.tab.h      */
        XGPObject *datafield;           /* see above        */
}stblentry;

typedef struct _htblentry {
                stblentry        *sp;
        struct _htblentry        *next;
}htblentry;

int             stblinit();
int             preload();
stblentry *stblget();
stblentry *lookup();

#endif _Objects_P_h
