#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <math.h>
#include "GraphP.h"
#include "pars.h"
#include "y.tab.h"

extern float return_float_point();
extern int return_int_point();

/********************************************************************/
/* July 1989    Dan Bennett         				    */
/* this procedure will check to see if the name has an associated   */
/* data field.  If so the name is in use otherwise the name is free */
/********************************************************************/

int Existing_Name(name)
    char      *name;

    {stblentry *t, *lookup();
       
       t = lookup(name);
       if (NULL == t) {
	  return (FALSE);
       }
       if (t->datafield == NULL)
	  return (FALSE);
       else
	  return (TRUE);
    }

/********************************************************************/
/* 11/10/89							    */
/* this one will return a pointer to the object.  Make sure you     */
/* know the object exists before you try to use it                  */
/********************************************************************/

XGPObject* find_object(name)
    char      *name;

    {stblentry *t, *lookup();

       t = lookup(name);
       return(t->datafield);
    }

int return_visibility(nid)
	int    nid;
	{
	    switch (nid) {
		case TINVISIBLE :
		   return (XtInvisible);
		   break;
		default :
		   return(XtVisible);
		   break;
	    }
	}

int return_justify(nid)
	int    nid;
	{
	    switch (nid) {
	        case TUP :
		   return(UP);
		   break;
	        case TDOWN :
		   return(DOWN);
		   break;
		case TCENTER :
		   return(CENTER);
		   break;
		case TLEFT :
		   return (LEFT);
		   break;
		case TRIGHT :
		default :
		   return(RIGHT);
		   break;
	    }
	}

/********************************************************************/
/* July 1989    Dan Bennett                                         */
/* This procedure will add the filled in data field for a new object*/
/* to the object's hash field                                       */
/********************************************************************/

void Add_Object (name, wid, data)
    char         *name;   /* the name of the new object  */
    GraphWidget  wid;     /* The widget the new object is part of */
    XGPObject    *data;   /* the data field for the new object   */

    {   XGPNameList   *new;
	stblentry  *t;

	t =  lookup(name);
	new = (XGPNameList*) XGPmalloc(wid,sizeof(XGPNameList));
	   new->object = data;
	   new->next = wid->graph.names;
	   wid->graph.names = new;
	   t->datafield = data;
     }

/********************************************************************/
/* JULY 1989  Dan Bennett      					    */
/* This procedure will create a new data node and fill in the       */
/* data points when a line is to be drawn.                          */
/********************************************************************/

XGPObject* AssignLineData(wid,tree)
	GraphWidget   wid;
	XGP_node     *tree;

	{ XGPObject  *temp = NULL;
	  XGP_node   *t1, *t2;

	  t1 = tree->child[0];
	  t2 = tree->child[1];

	  temp = (XGPObject*) XGPmalloc(wid,sizeof(XGPObject));
	  temp->Other.dimension = DeTreePoint(t1,
		  &temp->data.p_data.data.l_data.start_point);
	  (void)  DeTreePoint(t2,&temp->data.p_data.data.l_data.end_point);
	  temp->data.p_data.type = Line;
          return(temp);
	}

Numlist*   makepoly(wid,x,y,length,angle,ai,more)
	   GraphWidget  wid;
	   float   x, y, length, angle,ai;
	   int     more;

	   {
	   float     nx,ny;
           Numlist   *temp = NULL;
	   if (more <= 0) {
	      return (NULL);
	   } else {
	     temp = (Numlist*) XGPmalloc(wid,sizeof(Numlist));
	     temp->x = x;
	     temp->y = y;
	     nx = x + length * cos(angle*0.0174533);
	     ny = y + length * sin(angle*0.0174533);
	     temp->next = makepoly(wid,nx,ny,length,angle+ai,ai,more-1);
	     return(temp);
	   }
	}

XGPObject* AssignRegPolygonData(wid,tree)
	GraphWidget   wid;
        XGP_node  *tree;

	{
	 XGPObject  *temp = NULL;
          XGP_node   *t1, *t2;
	  float      x,y,length,angle;
	  int        sides;


          t1 = tree->child[0];
          t2 = tree->child[1];
          
	  temp = (XGPObject*) XGPmalloc(wid,sizeof(XGPObject));
	  x = return_float_point(t1->child[0]);
	  y = return_float_point(t1->child[1]);
	  length = return_float_point(t2->child[0]);
	  sides = return_int_point(t2->child[1]);
	  if (sides <= 2) {
	      error(wid,108,(char*)NULL);
	      return(NULL);
	  } else {
	      angle = (float) 360/(float) sides;
	      temp->data.p_data.data.y_data.points = 
			     makepoly(wid,x,y,length,angle,angle,sides);
	      temp->data.p_data.type = Polygon;
	      return(temp);
	  }
	}

/********************************************************************/
/* July 1989    Dan Bennett          			            */
/* This procedure will create and fill in x, y  cordinate           */
/* data for a point primitive                                       */
/********************************************************************/

XGPObject* AssignPointData(wid,tree)
	GraphWidget   wid;
	XGP_node  *tree;

	{
	  XGPObject   *temp = NULL;
          XGP_node   *t1;

          t1 = tree;
          temp = (XGPObject*) XGPmalloc(wid,sizeof(XGPObject));
	  temp->Other.dimension = DeTreePoint(t1->child[0],
		&temp->data.p_data.data.p_data.point);
	  temp->data.p_data.type =Point;
	  return(temp);
	}

XGPObject* AssignElipseData(wid,tree)
	GraphWidget   wid;
	XGP_node *tree;

	{
	  XGPObject *temp = NULL;
	  XGP_node  *t1,*t2;

	  t1 = tree->child[0];
	  t2 = tree->child[1];
	  temp = (XGPObject*) XGPmalloc(wid,sizeof(XGPObject));
	  temp->data.p_data.data.c_data.radius1 = 
				return_float_point(t2->child[0]);
	  temp->data.p_data.data.c_data.radius2 = 
				return_float_point(t2->child[1]);
	  temp->data.p_data.data.c_data.center.x =
				return_float_point(t1->child[0])+
				temp->data.p_data.data.c_data.radius1/2;
	  temp->data.p_data.data.c_data.center.y =
				return_float_point(t1->child[1])-
				temp->data.p_data.data.c_data.radius2/2;
	  temp->data.p_data.type = Elipse;
	  return(temp);
	}

XGPObject* AssignTextData(wid,tree)
	GraphWidget  wid;
	XGP_node      *tree;

	{
	XGPObject *temp = NULL;

	temp = (XGPObject*) XGPmalloc(wid,sizeof(XGPObject));
	temp->data.p_data.data.t_data.text =(char*) 
	XGPmalloc(wid,(strlen(tree->child[0]->sval)+1)*sizeof(char));
	strcat(temp->data.p_data.data.t_data.text,tree->child[0]->sval);
	temp->Other.dimension= DeTreePoint(tree->child[1],
	        &temp->data.p_data.data.t_data.point);
	temp->data.p_data.type = Text;
	return(temp);
        }

XGPObject*  AssignPolygonData(wid,tree)
	GraphWidget   wid;
	XGP_node      *tree;
	{
	XGPObject *temp = NULL;
	Numlist   *point,*list;
	XGP_node  *t,*r;
	int        tdtd;

	temp = (XGPObject*) XGPmalloc(wid,sizeof(XGPObject));
	if (tree->node_type == TPOLYGON3D) {
	    tdtd = TRUE;
	    temp->Other.dimension=T3D;
	} else {
	    tdtd = FALSE;
	    temp->Other.dimension=T2D;
	}
	list = NULL;
	t = tree->child[0];
	while (t != NULL) {
	    point = (Numlist*) XGPmalloc(wid,sizeof(Numlist));
            point->next = list;
	    if (tdtd == TRUE) {
	       r = t->child[0];
               point->z = return_float_point(t->child[1]);
	    } else {
	       r = t;
	       point->z = 0.0;
	    }
	    point->x = return_float_point(r->child[0]);
	    point->y = return_float_point(r->child[1]);
	    list = point;
	    t = t->sibling;
	}
	temp->data.p_data.data.y_data.points = list;
	temp->data.p_data.type = Polygon;
	return(temp);
	}

SquareSurfaceStruct* makerow(wid,tree,len,current)
	GraphWidget wid;
	XGP_node    *tree;
	int         len,current;
	 {
	 SquareSurfaceStruct   *here;

	 if (tree != NULL) {
	    here= (SquareSurfaceStruct*) 
		   XGPmalloc(wid,sizeof(SquareSurfaceStruct));
	    here->x = return_float_point(tree->child[0]->child[0]);
	    here->y = return_float_point(tree->child[0]->child[1]);
	    here->z = return_float_point(tree->child[1]);
	    if (len == current) {
	        here->fp = NULL;
	    } else {
		here->fp = makerow(wid,tree->sibling,len,current+1);
	    }
	    here->sp = NULL;
	    return(here);
	} else {
	    return(NULL);
	}
    }

XGP_node* movecurrent(current,rowlen,pos)
	XGP_node    *current;
	int         rowlen,pos;

	{
	  if (rowlen == pos) {
	     return(current);
	  } else {
	     if (current == NULL) {
	       return(current);
	     } else {
	       return(movecurrent(current->sibling,rowlen,pos+1));
	     }
	  }
	}

void combine(lastrow,next)
	SquareSurfaceStruct  *lastrow, *next;

	{
	    SquareSurfaceStruct   *foo1, *foo2;

	    foo1 = lastrow;
	    foo2 = next;

	    while(foo1 != NULL) {
		  foo1->sp = foo2;
		  if (foo2 != NULL) {
		      foo2 = foo2->fp;
		  }
		  foo1 = foo1->fp;
	    }
	}

XGPObject* AssignConeData(wid,tree)
	GraphWidget  wid;
	XGP_node    *tree;
	{
		XGPObject   *tempt;
                SquareSurfaceStruct *foo,*bar;
		float x,y,z,a,b,c,rad;
                float trad,radinc;
                int   i;
                float dx,dy,dz;
		
		Detree3DPoints(tree->child[0]->child[0],&x,&y,&z);
		Detree3DPoints(tree->child[0]->child[1],&a,&b,&c);
                rad = return_float_point(tree->child[1]);

		tempt = (XGPObject*) XGPmalloc(wid,sizeof(XGPObject));
		foo=(SquareSurfaceStruct*)CreateCircle(wid,x,y,z,a,b,c,rad,30);
                tempt->data.p_data.data.s_data.data=foo;

		radinc = rad / 10;
		dx = (a-x)/10;
		dy = (b-y) / 10;
		dz = (c-z) / 10;

		for (i=1;i<= 10 ;i++) {
		    bar = (SquareSurfaceStruct*)CreateCircle
			   (wid,x+i*dx,y+i*dy,z+i*dz,a,b,c,rad-i*radinc,30);
		    combine(foo,bar);
		    foo = bar;
		}
                tempt->Other.dimension = T3D;
	        tempt->data.p_data.type = Surface;
		return(tempt);
	}

XGPObject* AssignCylenderData(wid,tree)	
	GraphWidget  wid;
	XGP_node    *tree;
	{
		XGPObject   *tempt;
                SquareSurfaceStruct *foo,*bar;
		float x,y,z,a,b,c,rad;
                float trad,radinc;
                int   i;
                float dx,dy,dz;
		
		Detree3DPoints(tree->child[0]->child[0],&x,&y,&z);
		Detree3DPoints(tree->child[0]->child[1],&a,&b,&c);
                rad = return_float_point(tree->child[1]);

		tempt = (XGPObject*) XGPmalloc(wid,sizeof(XGPObject));
		foo=(SquareSurfaceStruct*)CreateCircle(wid,x,y,z,a,b,c,rad,30);
                tempt->data.p_data.data.s_data.data=foo;

		dx = (a-x)/10;
		dy = (b-y) / 10;
		dz = (c-z) / 10;

		for (i=1;i<= 10 ;i++) {
		    bar = (SquareSurfaceStruct*)CreateCircle
			   (wid,x+i*dx,y+i*dy,z+i*dz,a+dx,b+dy,c+dz,rad,30);
		    combine(foo,bar);
		    foo = bar;
		}
                tempt->Other.dimension = T3D;
	        tempt->data.p_data.type = Surface;
		return(tempt);
	}

#define degrees_to_radians   0.0174533

XGPObject* AssignSphereData(wid,tree)	
	GraphWidget   wid;
	XGP_node    *tree;
	{
		XGPObject   *tempt;
                SquareSurfaceStruct *foo,*bar;
		float x,y,z,a,b,c,rad;
                float trad,radinc;
                int   i;
                float dx,dy,dz;

		Detree3DPoints(tree->child[0],&x,&y,&z);
                rad = return_float_point(tree->child[1]);

		tempt = (XGPObject*) XGPmalloc(wid,sizeof(XGPObject));
    	      foo=(SquareSurfaceStruct*)CreateCircle
		     (wid,x,y,z,x+10.0,y,z,rad,30);
                tempt->data.p_data.data.s_data.data=foo;

		for (i=10;i<= 180;i=i+10) {
		    bar = (SquareSurfaceStruct*)CreateCircle
			   (wid,x,y,z,(float)x+10*cos(i*degrees_to_radians),
			     (float) y+10*sin(i*degrees_to_radians),z,rad,30);
		    combine(foo,bar);
		    foo = bar;
		}
                tempt->Other.dimension = T3D;
	        tempt->data.p_data.type = Surface;
		return(tempt);
	}

XGPObject* AssignCircleData(wid,tree)
	GraphWidget   wid;
	XGP_node    *tree;
	{
		XGPObject   *tempt;
                SquareSurfaceStruct *foo;
		float x,y,z,a,b,c,rad;

		Detree3DPoints(tree->child[0]->child[0],&x,&y,&z);
		Detree3DPoints(tree->child[0]->child[1],&a,&b,&c);
                rad = return_float_point(tree->child[1]);

		tempt = (XGPObject*) XGPmalloc(wid,sizeof(XGPObject));
		foo = (SquareSurfaceStruct*)CreateCircle
		       (wid,x,y,z,a,b,c,rad,50);
                tempt->data.p_data.data.s_data.data=foo;
                tempt->Other.dimension = T3D;
	        tempt->data.p_data.type = Surface;
		return(tempt);
        }

XGPObject* AssignSurfaceData(wid,tree)
	GraphWidget   wid;
	XGP_node    *tree;
	{
		int         rowlen,temp;
                XGP_node    *current;
                SquareSurfaceStruct *next,*lastrow;
                XGPObject   *tempt;

		rowlen = return_int_point(tree->child[0]);
                tempt = (XGPObject*) XGPmalloc(wid,sizeof(XGPObject));
		tempt->data.p_data.data.s_data.rows = rowlen;
                   /* start through the list of points and make put them
             		      in a list.*/
		   lastrow = makerow(wid,tree->child[1],rowlen,1);
		   tempt->data.p_data.data.s_data.data=lastrow;
		   current= tree->child[1];
		   current= movecurrent(current,rowlen,0);
		   while (current != NULL) {
		       next = makerow(wid,current,rowlen,1);
		       combine(lastrow,next);
		       lastrow = next;
		       current = movecurrent(current,rowlen,0);
		   }
		   tempt->Other.dimension=T3D;
		   tempt->data.p_data.type =Surface;
		return(tempt);
	}

XGPObject*  AssignPolylineData(wid,tree)
	GraphWidget   wid;
	XGP_node      *tree;
	{
	XGPObject *temp = NULL;
	Numlist   *point,*list;
	XGP_node  *t,*r;

        int        tdtd;

        temp = (XGPObject*) XGPmalloc(wid,sizeof(XGPObject));

        if (tree->node_type == TPOLYLINE3D) {
            tdtd = TRUE;
            temp->Other.dimension=T3D;
        } else {
            tdtd = FALSE;
            temp->Other.dimension=T2D;
        }
	
	t = tree->child[0];
	list = NULL;
	while (t != NULL) {
	    point = (Numlist*) XGPmalloc(wid,sizeof(Numlist));
            point->next = list;
            if (tdtd == TRUE) {
               r = t->child[0];
               point->z = return_float_point(t->child[1]);
            } else {
               r = t;
               point->z = 0.0;
            }
	    point->x = return_float_point(r->child[0]);
	    point->y = return_float_point(r->child[1]);
	    list = point;
	    t = t->sibling;
	}
	temp->data.p_data.data.pl_data.points = list;
	temp->data.p_data.type = Polyline;
	return(temp);
	}

void AssignOtherAttribs(wid,data,tree)
	GraphWidget wid;
	XGPObject   *data;
	XGP_node    *tree;
	{
	   char        *foo;
	   if (tree != NULL) {
	       switch (tree->node_type) {
		 case TVISIBILITY :
		 case THIDDEN :
		     if (tree->child[0]->node_type == TVISIBLE) {
			 data->Other.Hidden = XtVisible;
		     } else {
			 data->Other.Hidden = XtInvisible;
		     }
		     break;
		 case TLINEST :
		      data->Other.linestyle = tree->child[0]->node_type;
		      break;
		 case TFONT :
		      data->Other.fontstyle = tree->child[0]->node_type;
		      break;
		 case TLINEWID :
		     data->Other.linewidth = return_int_point(tree->child[0]);
		     break;
		 case TCOLOR :
		     foo = tree->child[0]->sval;
		     AssignColorField(wid,foo,&data->Other.fgcolor);
		     break;
		 case TROTATE :
		     MakeRotate(tree,data->Other.transformation);
		     break;
                 case TTRANSLATE :
                     MakeTranslate(tree,data->Other.transformation);
                     break;
                 case TSCALE :
                     MakeScale(tree,data->Other.transformation);
                     break;		
                case TTRANSFORM :
                     MakeTransform(tree,data->Other.transformation);
                     break;
		 case TLINE :
		 case TPOINT:
		 case TCIRCLE:
		 case TCONE:
		 case TCYLENDER :
		 case TSPHERE:
		 case TELIPSE:
		 case TPOLYGON:
		 case TPOLYGON3D :
		 case TSURFACE :
		 case TREGPOLY:
		 case TPOLYLINE:
		 case TPOLYLINE3D :
		 case TTEXT:
		     break;
		 default :
		     error(wid,161,(char*)NULL);
		     break;
	       }
	       AssignOtherAttribs(wid,data,tree->sibling);
	   }
	}

void AssignDefaultAttribs(wid,data)
	GraphWidget wid;
	XGPObject   *data;

	{
	  int   i,j;

	  data->Other.Hidden = XtVisible;
	  for (i=0;i<4;i++) {
	     for (j=0;j<4;j++) {
	       data->Other.transformation[i][j] = 0;
	     }
	  }
	  data->Other.transformation[1][1] = 1;
	  data->Other.transformation[2][2] = 1;
	  data->Other.transformation[0][0] = 1;
	  data->Other.transformation[3][3] = 1;
          data->Other.linewidth = 0;
	  data->Other.linestyle = TSOLID;
	  data->Other.fontstyle = TREGULAR;
	  data->Other.fgcolor = NULL;
	}


XGPObject* FindCritNode(wid,tree)
	GraphWidget wid;
        XGP_node   *tree;

        {
          XGPObject   *data;

          if (tree == NULL) {
              return(NULL);
          } else {
          switch (tree->node_type) {
             case TLINE :
                 data = AssignLineData(wid,tree);
                 return(data); 
 		 break;
             case TPOINT :
                 data = AssignPointData(wid,tree);
                 return(data);
                 break;
             case TELIPSE :
                 data = AssignElipseData(wid,tree);
		 data->Other.dimension=T2D;
                 return(data);
                 break;
	     case TCIRCLE:
		 data = AssignCircleData(wid,tree);
		 data->Other.dimension=T3D;
		 return(data);
                 break;
	     case TSPHERE :
		 data = AssignSphereData(wid,tree);
		 return(data);
		 break;
	     case TCONE :
		 data = AssignConeData(wid,tree);
		 return(data);
		 break;
	     case TCYLENDER : 
		 data = AssignCylenderData(wid,tree);
		 return(data);
		 break;
             case TPOLYGON :
	     case TPOLYGON3D :
                 data = AssignPolygonData(wid,tree);
                 return(data);
                 break;
             case TSURFACE :
		 data= AssignSurfaceData(wid,tree);
		 return(data);
		 break;
             case TREGPOLY :
		 data = AssignRegPolygonData(wid,tree);
		 data->Other.dimension=T2D;
                 return(data);
                 break;
	     case TPOLYLINE3D :
             case TPOLYLINE :
		 data = AssignPolylineData(wid,tree);
                 return(data);
                 break;
	     case TTEXT :
		 data = AssignTextData(wid,tree);
		 return(data);
		 break;
             default :
                 return(FindCritNode(wid,tree->sibling));
                 break;
          }
         }
        }

XGPObject* MakePrimitive (wid,name,tree)
	GraphWidget wid;
	char       *name;
	XGP_node   *tree;

	{
	 XGPObject    *data= NULL;

	   if (Existing_Name(name)){
	       error(wid,105,name);
	       return(NULL);
	   } else {
	       data  = FindCritNode(wid,tree);
	       if (data  == NULL) {
		 error(wid,106,name);
		 return(NULL);
               } else {
		 AssignDefaultAttribs(wid,data);
		 AssignOtherAttribs(wid,data,tree);
		 data->type = XGPPrimitive;
		 return(data);
	       }
	   }
	}
