/*********************************************************************/
/* the following are all the language constructs                     */
/*********************************************************************/
%token	TDRAWPRIM TCOMBINE TMOVE TCOPY
%token	TSET TCHANGE TINQUIRE
%token	TREMOVE TQUIT TSLEEP THARDCOPY TRESET
%token	TIDENT TFCONST TICONST TLP TRP TSCONST TCOMMENT

/*********************************************************************/
/* These are file commands as requested by Dr. Rothstein             */
/*********************************************************************/

%token  TEXECUTE TSAVECOMM TSAVECOMMSTOP TALIAS TSAVE

/*********************************************************************/
/* the following are critical primitive attributes                   */
/*********************************************************************/

%token	TLINE TPOINT TCIRCLE TELIPSE TPOLYGON TREGPOLY TPOLYLINE TPOINT3D
%token  TTEXT TPOLYGON3D TSURFACE TCONE TSPHERE TCYLENDER TPOLYLINE3D
%token  TLINE3D THLHS

/*********************************************************************/
/* the following are the reference tokens                            */
/*********************************************************************/

%token TVISIBILITY TAUTOADJ TWIDTH TXWIDTH TYWIDTH TCENTER
%token TRSTYLE TRXSTYLE TRYSTYLE

%token TLABVIS TLABXVIS TLABYVIS TLABFONT TLABXFONT TLABYFONT
%token TLABPRE TLABXPRE TLABYPRE TLABSKIP TLABXSKIP TLABYSKIP
%token TLABJUST TLABXJUST TLABYJUST TLABRANGE TLABXRANGE TLABYRANGE

%token TTICKVIS TTICKXVIS TTICKYVIS TTICKSCALE TTICKXSCALE
%token TTICKYSCALE TTICKRANGE TTICKXRANGE TTICKYRANGE

%token TXAXIS TYAXIS TZAXIS TTICCOLOR TLABCOLOR
%token TVIEWDIM TVIEWCOR

/********************************************************************/
/* these are other world attributes                                 */
/********************************************************************/

%token TVIEWPORT TMINMAX T2D T3D TREPLACE TAPPEND
%token TEYEPOS TFOCPOS TDISTFROMI TVIEWSIZE TVIEWROT
%token TCOLORMAP TPUBLIC TPRIVATE TCENTERED
%token T2DONLY T3DONLY TDIMENSION

/********************************************************************/
/*    These are other attributes   mostly common attributes         */
/********************************************************************/

%token THIDDEN TROTATE TTRANSLATE TSCALE TIDENTITY TTRANSFORM

/********************************************************************/
/*   These are line attributes                                      */
/********************************************************************/

%token TLINEST TLINEWID TSOLID TDASHED TDOUBLE TNONE TDEFAULT TCOLOR

/********************************************************************/
/* These are for the control of the text and fonts                  */
/********************************************************************/

%token TFONT TREGULAR TITALIC TBOLD TGREEK TLOADFONT
%token TVISIBLE TINVISIBLE TUP TDOWN TLEFT TRIGHT TTRUE TFALSE

/********************************************************************/
/* These are TOLKIENS (JRR) for hardcopy                            */
/********************************************************************/

%token TOUTFORM TOUTFILE TSIZEMES TDPI TPICTLEN TPICTWID
%token TINCH TCENT TPAINTJET TLASERJET TTEXSPEC TOUTPUT

/********************************************************************/
/* These are SUI mode  (on or off)                                  */
/********************************************************************/
%token TSUIMODE TON TOFF
%token TREDRAW
%token TOBJECT
%{
/*********************************************************************/
/* The shell for this file and for all of the parser scanner was     */
/* was provided by Sanjiva Weerawarana and was origionally developed */
/* for GENCRAY  his Masters Thesis                                   */
/*********************************************************************/

#include	<stdio.h>
#include	"pars.h"

XGP_node	*XGP_command;

%}

%union	{
	XGP_node	*_node;
	char		*_sval;
	int		_ival;
	float		_fval;
}

%start command 

%type <_ival>	TICONST
%type <_fval>	TFCONST
%type <_sval>	TIDENT TSCONST
%type <_node>	command command1 
%type <_node>	create_command change_command destroy_command utility_command
%type <_node>   reference_attribute label_attrib tick_attrib
%type <_node>	name name_list string viewport viewbox_attribute
%type <_node>	attribute attr_list numlist point fontstyle transform
%type <_node>	num fnum inum visibility justify bool style tmode
%type <_node>   numlist3d point3d axis3d_attribute hard_attribute
%type <_node>   outputformat cmaptype inquire_attrib inquire_list
%type <_node>   object_attribute critical_2D general_attribute
%type <_node>   critical_3D world_attribute on_off viewing_attribute
%type <_node>   translate_attribute full_inquire_list dimension
%%
command	: command1
	{
		XGP_command = $1;
		return (0);
	}
	|TCOMMENT 
	{
		XGP_command = NULL;
		printf("Comment Found %d \n",xgp_paran_ct);
		return (1);
	}
	;

command1: create_command
	| change_command
	| destroy_command
	| utility_command
	| TLP error TRP
	{
		yyclearin;
		yyerrok;
		resync ();
		$$ = NULL;
	}
	;

create_command: TLP TDRAWPRIM name attr_list TRP
	{
		$$ = newnode (TDRAWPRIM, $3, $4);
	}
	| TLP TCOMBINE name name_list TRP
	{
		$$ = newnode (TCOMBINE, $3, $4);
	}
	| TLP TCOPY name name TRP
	{
		$$ = newnode (TCOPY, $3, $4);
	}
	;

change_command: TLP TSET name attr_list TRP
	{
		$$ = newnode (TSET, $3, $4);
	}
	| TLP TMOVE name name TRP
	{
		$$ = newnode (TMOVE, $3, $4);
	}
	| TLP TCHANGE name attr_list TRP
	{
		$$ = newnode (TCHANGE, $3, $4);
	}
	;

destroy_command: TLP TREMOVE name_list TRP
	{
		$$ = newnode (TREMOVE, $3, NULL);
	}
	;

utility_command : TLP TQUIT TRP
	{
		$$ = newnode(TQUIT, NULL, NULL);
	}
	| TLP TINQUIRE name inquire_list TRP
	{
		$$ = newnode(TINQUIRE, $3, $4);
	}
	| TLP TINQUIRE TRP
	{
		$$ = newnode(TINQUIRE, NULL, NULL);
	}
	| TLP THARDCOPY TRP
	{
		$$ = newnode(THARDCOPY, NULL, NULL);
	}
	| TLP TSLEEP num TRP
	{
		$$ = newnode(TSLEEP, $3, NULL);
	}
	| TLP TEXECUTE string TRP
	{
		$$ = newnode (TEXECUTE, $3, NULL);
	}
	| TLP TSAVE string TRP
	{
		$$ = newnode (TSAVE, $3, NULL);
	}
	| TLP TSAVECOMM string TRP
	{
		$$ = newnode (TSAVECOMM, $3, NULL);
	}
	| TLP TSAVECOMMSTOP TRP
	{
		$$ = newnode (TSAVECOMMSTOP, NULL, NULL);
	}
	| TLP TRESET TRP
	{
		$$ = newnode (TRESET, NULL, NULL);
	}
	| TALIAS 
	{
		$$ = newnode(TALIAS, NULL, NULL);
	}
	;

name	: TIDENT
	{
		$$ = newnode (TIDENT, NULL, NULL);
		$$->sval = $1;
	}
	;

name_list: name name_list
	{
		$$ = makesibs($1,$2);
	}
	| name
	;

attr_list: attribute attr_list
	{
		$$ = makesibs($1,$2);
	}
	| attribute
	;

attribute: translate_attribute
	| TLP TLOADFONT fontstyle string TRP
	{
		$$ = newnode (TLOADFONT, $3, $4);
	}
        | object_attribute
	| general_attribute
	| reference_attribute
	| axis3d_attribute
	| viewbox_attribute
	| hard_attribute
	| world_attribute
	;

world_attribute : TLP TREDRAW on_off TRP
	{
		$$ = newnode(TREDRAW, $3, NULL);
	}
	| TLP THLHS on_off TRP
	{
		$$ = newnode(THLHS, $3, NULL);
	}
	| TLP TSUIMODE on_off TRP
	{
		$$ = newnode(TSUIMODE, $3, NULL);
	}
        | TLP TCOLORMAP cmaptype TRP
        {
                $$ = newnode (TCOLORMAP, $3, NULL);
        }
	| TLP TDIMENSION dimension TRP
        {
                $$ = newnode (TDIMENSION, $3, NULL);
        }
	| viewing_attribute
        ;

viewing_attribute : TLP TVIEWPORT viewport TRP
        {
                $$ = newnode (TVIEWPORT, $3, NULL);
        }
        | TLP TEYEPOS point3d TRP
        {
                $$ = newnode(TEYEPOS, $3, NULL);
        }
        | TLP TFOCPOS TCENTERED TRP
        {
                $$ = newnode(TFOCPOS, NULL, NULL);
        }
        | TLP TFOCPOS point3d TRP
        {
                $$ = newnode(TFOCPOS, $3, NULL);
        }
        | TLP TDISTFROMI num TRP
        {
                $$ = newnode(TDISTFROMI, $3, NULL);
        }
        | TLP TVIEWSIZE num num TRP
        {
                $$ = newnode(TVIEWSIZE,$3, $4);
        }
        | TLP TVIEWROT num TRP
        {
                $$ = newnode(TVIEWROT, $3, NULL);
        }
	;

general_attribute : TLP THIDDEN visibility TRP
        {
                $$ = newnode (TVISIBILITY, $3, NULL);
        }
        | TLP TVISIBILITY visibility TRP
        {
                $$ = newnode (TVISIBILITY, $3, NULL);
        }
        | TLP TCOLOR string TRP
        {
                $$ = newnode (TCOLOR, $3, NULL);
        }
	| TLP TCOLOR TDEFAULT TRP
	{
		$$ = newnode(TCOLOR, NULL, NULL)
	}
        | TLP TLINEWID num TRP
        {
                $$ = newnode (TLINEWID, $3, NULL);
        }
        | TLP TLINEST style TRP
        {
                $$ = newnode (TLINEST, $3, NULL);
        }
        | TLP TFONT fontstyle TRP
        {
                $$ = newnode (TFONT, $3, NULL);
        }
	;

translate_attribute : TLP TTRANSFORM tmode transform TRP
        {
                $$ = newnode (TTRANSFORM, $3, $4);
        }
        | TLP TROTATE num num num TRP
        {
                XGP_node        *p1, *p2;

                p1 = newnode (TPOINT, $3, $4);
                p2 = newnode (TPOINT, $5, $5);
                $$ = newnode (TROTATE, p1, p2);
        }
        | TLP TROTATE point3d point3d num TRP
        {
                XGP_node        *p1, *p2;

                p1 = newnode (TPOINT3D, $3, $4);
                p2 = newnode (TPOINT3D, $5, $5);
                $$ = newnode (TROTATE, p1, p2);
        }
        | TLP TTRANSLATE num num num TRP
        {
                XGP_node        *p1;

                p1 = newnode(TPOINT, $3, $4);
                $$ = newnode(TTRANSLATE, p1, $5);
        }
        | TLP TTRANSLATE num num TRP
        {
                $$ = newnode(TTRANSLATE, $3, $4);
        }
        | TLP TSCALE num num num num num num TRP
        {
                XGP_node *p1, *p2, *p3, *p4;

                p2 = newnode(TPOINT, $3, $4);
                p1 = newnode(TPOINT, p2, $5);
                p4 = newnode(TPOINT, $6, $7);
                p3 = newnode(TPOINT, p4, $8);
                $$ = newnode(TSCALE, p1, p3);
        }
        | TLP TSCALE num num num num TRP
        {
                XGP_node        *p1, *p2;

                p1 = newnode (TPOINT, $3, $4);
                p2 = newnode (TPOINT, $5, $6);
                $$ = newnode (TSCALE, p1, p2);
        }
        ;

object_attribute : critical_2D
	| critical_3D
	;

critical_2D : TLP TTEXT string point TRP
	{
		$$ = newnode (TTEXT, $3, $4);
	}
	| TLP TLINE point point TRP
	{
		$$ = newnode (TLINE, $3, $4);
	}
        | TLP TPOINT point TRP
	{
		$$ = newnode(TPOINT, $3, NULL);
	}
	| TLP TPOLYLINE numlist TRP
        {
                $$ = newnode (TPOLYLINE, $3, NULL);
        }
        | TLP TPOLYGON  numlist TRP
        {
                $$ = newnode (TPOLYGON, $3, NULL);
        }
        | TLP TREGPOLY point point TRP
        {
                $$ = newnode (TREGPOLY, $3, $4);
        }
        | TLP TCIRCLE num num num TRP
        {
                XGP_node      *p1, *p2;

                p1 = newnode (TPOINT, $3, $4);
                p2 = newnode (TPOINT, $5, $5);
                $$ = newnode (TELIPSE, p1, p2);
        }
	;

critical_3D : TLP TTEXT string point3d TRP
	{
		$$ = newnode(TTEXT, $3, $4);
	}
	| TLP TLINE3D point3d point3d TRP
        {
		$$ = newnode (TLINE, $3, $4);
	}
        | TLP TPOINT3D  point3d TRP
        {
                $$ = newnode(TPOINT, $3, NULL);
        }
        | TLP TCIRCLE point3d point3d num TRP
        {
                XGP_node        *p1;

                p1 = newnode (TPOINT, $3, $4);
                $$ = newnode (TCIRCLE, p1, $5);
        }
        | TLP TCYLENDER point3d point3d num TRP
        {
                XGP_node       *p1;

                p1 = newnode(TPOINT, $3, $4);
                $$ = newnode(TCYLENDER, p1, $5);
        }
        | TLP TCONE point3d point3d num TRP
        {
                XGP_node       *p1;

                p1 = newnode(TCONE, $3, $4);
                $$ = newnode(TCONE, p1, $5);
        }
        | TLP TSPHERE point3d num TRP
        {
                $$ = newnode(TSPHERE, $3, $4);
        }
        | TLP TELIPSE num num num num TRP
        {
                XGP_node      *p1, *p2;

                p1 = newnode (TPOINT, $3, $4);
                p2 = newnode (TPOINT, $5, $6);
                $$ = newnode (TELIPSE, p1, p2);
        }
        | TLP TPOLYLINE3D numlist3d TRP
        {
                $$ = newnode(TPOLYLINE3D, $3, NULL);
        }
        | TLP TPOLYGON3D numlist3d TRP
        {
                $$ = newnode(TPOLYGON3D, $3, NULL);
        }
        | TLP TSURFACE num numlist3d TRP
        {
                $$ = newnode(TSURFACE, $3, $4);
        }
	;

inquire_list: /* empty */
	{
		$$ = NULL;
	}
	| full_inquire_list
	;


full_inquire_list: inquire_attrib
	| inquire_attrib full_inquire_list 
	{
		$$ = makesibs($1, $2);
	}
	;

inquire_attrib: TLP TOUTPUT TRP
	{
		$$ = newnode(TOUTPUT, NULL, NULL);
	}
	| TLP TEYEPOS TRP
	{
		$$ = newnode(TEYEPOS, NULL, NULL);
	}
	| TLP TFOCPOS TRP
	{
		$$ = newnode(TFOCPOS, NULL, NULL);
	}
	| TLP TDISTFROMI TRP
	{
		$$ = newnode(TDISTFROMI, NULL, NULL);
	}
	| TLP TVIEWSIZE TRP
	{
		$$ = newnode(TVIEWSIZE, NULL, NULL);
	}
	| TLP TOBJECT TRP
	{
		$$ = newnode(TOBJECT, NULL, NULL);
	}
	| TLP TVIEWPORT TRP
	{
		$$ = newnode(TVIEWPORT, NULL, NULL);
	}
	;

hard_attribute : TLP TOUTFILE string TRP
	{
		$$ = newnode(TOUTFILE, $3, NULL);
	}
	| TLP TPICTLEN num TRP
	{
		$$ = newnode(TPICTLEN, $3, NULL);
	}
	| TLP TPICTWID num TRP
	{
		$$ = newnode(TPICTWID, $3, NULL);
	}
        | TLP TDPI num TRP
        {
                $$ = newnode(TDPI, $3, NULL);
        }
        | TLP TOUTFORM outputformat TRP
        {
                $$ = newnode(TOUTFORM, $3, NULL);
        }
        ;

axis3d_attribute : TLP TXAXIS point point num TRP

	{
		XGP_node  *p1;

		p1 = newnode(TPOINT, $4, $5);
		$$ = newnode(TXAXIS, $3, p1);
	}
	| TLP TYAXIS point point num TRP
	{
                XGP_node  *p1;

                p1 = newnode(TPOINT, $4, $5);
                $$ = newnode(TYAXIS, $3, p1);
	}
	| TLP TZAXIS point point num TRP
	{
                XGP_node  *p1;

                p1 = newnode(TPOINT, $4, $5);
                $$ = newnode(TZAXIS, $3, p1);
	}
	;

viewbox_attribute : TLP TVIEWDIM point3d  point3d  TRP
	{
		$$ = newnode(TVIEWDIM, $3, $4);
	}
	| TLP TVIEWCOR point3d  point3d  TRP
	{
		$$ = newnode(TVIEWCOR, $3, $4);
	}
	;

/* the following takes care of the reference line stuff (Next three things) */
/* 10/11/89								    */

reference_attribute : TLP TAUTOADJ bool TRP
	{
		$$ = newnode (TAUTOADJ, $3, NULL);
	}
	| TLP TWIDTH num TRP
        {
                $$ = siblings(TXWIDTH, TYWIDTH, $3, NULL);
        }
	| TLP TXWIDTH num TRP
	{
		$$ = newnode (TXWIDTH, $3, NULL);
        }
	| TLP TYWIDTH num TRP
	{
		$$ = newnode (TYWIDTH, $3, NULL);
	}
	| TLP TCENTER num num TRP
	{
		$$ = newnode (TCENTER, $3, $4);
	}
	| TLP TRSTYLE style TRP
	{
		$$ =siblings(TRXSTYLE, TRYSTYLE, $3, NULL);
	}
        | TLP TRXSTYLE style TRP
	{
		$$ = newnode (TRXSTYLE, $3, NULL);
	}
	| TLP TRYSTYLE style TRP
	{
		$$ = newnode (TRYSTYLE, $3, NULL);
	}
	| label_attrib
        | tick_attrib
	;

label_attrib : TLP TLABVIS visibility TRP
	{
		$$ = siblings (TLABXVIS, TLABYVIS, $3, NULL);
	}
	| TLP TLABXVIS visibility TRP
	{
		$$ = newnode (TLABXVIS, $3, NULL);
	}
        | TLP TLABCOLOR string TRP
	{
		$$ = newnode (TLABCOLOR, $3, NULL);
	}
	| TLP TLABYVIS visibility TRP
	{
		$$ = newnode (TLABYVIS, $3, NULL);
	}
	| TLP TLABFONT fontstyle TRP
	{
		$$ = siblings (TLABXFONT, TLABYFONT, $3, NULL);
	}
	| TLP TLABXFONT fontstyle TRP
	{
		$$ = newnode (TLABXFONT, $3, NULL);
	}
	| TLP TLABYFONT fontstyle TRP
	{
		$$ = newnode (TLABYFONT, $3, NULL);
	}
	| TLP TLABPRE num num TRP
	{
		$$ = siblings(TLABXPRE, TLABYPRE, $3, $4);
	}
	| TLP TLABXPRE num num TRP
	{
		$$ = newnode(TLABXPRE, $3, $4);
	}
	| TLP TLABYPRE num num TRP
        {
		$$ = newnode(TLABYPRE, $3, $4);
	}
	| TLP TLABSKIP num TRP
	{
		$$ = siblings(TLABXSKIP, TLABYSKIP, $3, NULL);
	}
	| TLP TLABXSKIP num TRP
	{
		$$ = newnode(TLABXSKIP, $3, NULL);
	}
	| TLP TLABYSKIP num TRP
	{
		$$ = newnode(TLABYSKIP, $3, NULL);
	}
	| TLP TLABJUST justify TRP
	{      
		$$ = siblings(TLABXJUST, TLABYJUST, $3, NULL);
	}
	| TLP TLABXJUST justify TRP
	{
		$$ =  newnode (TLABXJUST, $3, NULL);
	}
	| TLP TLABYJUST justify TRP
	{
		$$ =  newnode (TLABYJUST, $3, NULL);
	}
	| TLP TLABRANGE num num TRP
        {      
		$$ = siblings(TLABXRANGE, TLABYRANGE, $3, $4); 
	}
	| TLP TLABXRANGE num num TRP
        {
                $$ = newnode (TLABXRANGE, $3, $4);
        }
	| TLP TLABYRANGE num num TRP
	{
		$$ = newnode (TLABYRANGE, $3, $4);
	}
	;

tick_attrib : TLP TTICKVIS visibility TRP
	{
		$$ = siblings(TTICKXVIS, TTICKYVIS, $3, NULL);
	}
	| TLP TTICKXVIS visibility TRP
	{
		$$ = newnode (TTICKXVIS, $3, NULL);
	}
	| TLP TTICKYVIS visibility TRP
	{
		$$ = newnode (TTICKYVIS, $3, NULL);
	}
	| TLP TTICKSCALE num TRP
	{
		$$ = siblings(TTICKXSCALE, TTICKYSCALE, $3, NULL);
	}
	| TLP TTICKXSCALE num TRP
	{
		$$ = newnode (TTICKXSCALE, $3, NULL);
	}
	| TLP TTICKYSCALE num TRP
	{
		$$ = newnode (TTICKYSCALE, $3, NULL);
	}
	| TLP TTICKRANGE num num TRP
	{
		$$ = siblings(TTICKXRANGE, TTICKYRANGE, $3, $4);
	}
	| TLP TTICKXRANGE num num TRP
	{
		$$ = newnode (TTICKXRANGE, $3, $4);
	}
	| TLP TTICKYRANGE num num TRP
	{
		$$ = newnode (TTICKYRANGE, $3, $4);
	}
        | TLP TTICCOLOR string TRP
	{
		$$ = newnode (TTICCOLOR, $3, NULL);
	}
        ;

tmode : TAPPEND
        {
		$$ = newnode(TAPPEND, NULL, NULL);
	}
	| TREPLACE
	{
		$$ = newnode(TREPLACE, NULL, NULL);
	}
	;

viewport: point point
	{
	    $$ = newnode(TPOINT, $1, $2);
	}
	| TDEFAULT
	{
	    $$ = newnode(TDEFAULT, NULL, NULL);
	}
	| TMINMAX
	{
	    $$ = newnode(TMINMAX, NULL, NULL)
	}
        ;

transform : TIDENTITY
	{ 
		$$ = newnode(TIDENTITY, NULL, NULL);
	}
	| num num num num num num num num num
	{
		XGP_node    *p0,*p1,*p2,*p3,*p4,*p5,*p6;

		p6 = newnode(TPOINT, $8, $9);
		p5 = newnode(TPOINT, $6, $7);
		p4 = newnode(TPOINT, $4, $5);
		p3 = newnode(TPOINT, $2, $3);
		p2 = newnode(TPOINT, $1, p6);
		p1 = newnode(T2D, p4, p5);
		p0 = newnode(T2D, p2, p3);
                $$ = newnode(T2D, p0, p1); 
	}
	| num num num num num num num num num num num num num num num num
	{
		XGP_node *p0,*p1,*p2,*p3,*p4,*p5,*p6,*p7,*p8,*p9,
			 *p10,*p11,*p12,*p13,*p14;

		p0 = newnode(TPOINT, $1, $2);
		p1 = newnode(TPOINT, $3, $4);
		p2 = newnode(TPOINT, $5, $6);
		p3 = newnode(TPOINT, $7, $8);
		p4 = newnode(TPOINT, $9, $10);
		p5 = newnode(TPOINT, $11, $12);
		p6 = newnode(TPOINT, $13, $14);
		p7 = newnode(TPOINT, $15, $16);
		p8 = newnode(T3D, p0, p1);
		p9 = newnode(T3D, p2, p3);
		p10 = newnode(T3D, p4, p5);
		p11 = newnode(T3D, p6, p7);
		p12 = newnode(T3D, p8, p9);
		p13 = newnode(T3D, p10, p11);
		$$ = newnode(T3D, p12, p13);
	}
	;

on_off: TON 
	{
		$$ = newnode (TON, NULL, NULL);
	}
	| TOFF 
	{
		$$ = newnode (TOFF, NULL, NULL);
	}
	;

visibility: TVISIBLE
	{
		$$ = newnode (TVISIBLE, NULL, NULL);
	}
	| TINVISIBLE
	{
		$$ = newnode (TINVISIBLE, NULL, NULL);
	}
	;

style : TSOLID
	{
		$$ = newnode (TSOLID, NULL, NULL);
	}
	| TDOUBLE
	{
		$$ = newnode (TDOUBLE, NULL, NULL);
	}
	| TDASHED
	{
		$$ = newnode (TDASHED, NULL, NULL);
	}
	;

fontstyle : TREGULAR
	{
		$$ = newnode (TREGULAR, NULL, NULL);
	}
	| TITALIC
	{
		$$ = newnode (TITALIC, NULL, NULL);
	}
	| TBOLD 
	{
		$$ = newnode (TBOLD, NULL, NULL);
	}
	| TGREEK
	{
		$$ = newnode (TGREEK, NULL, NULL);
	}
	;

justify: TUP
	{
		$$ = newnode (TUP, NULL, NULL);
	}
	| TDOWN
	{
		$$ = newnode (TDOWN, NULL, NULL);
	}
	| TCENTER
	{
		$$ = newnode (TCENTER, NULL, NULL);
	}
	| TLEFT
	{
		$$ = newnode (TLEFT, NULL, NULL);
	}
	| TRIGHT
	{
		$$ = newnode (TRIGHT, NULL, NULL);
	}
	;

bool    : TTRUE
	{
		$$ = newnode(TTRUE, NULL, NULL);
	}
	| TFALSE
	{
		$$ = newnode(TFALSE, NULL, NULL);
	}
	;

cmaptype : TPUBLIC 
	{
		$$ = newnode(TPUBLIC, NULL, NULL);
	}
	| TPRIVATE 
	{
		$$ = newnode(TPRIVATE, NULL, NULL);
	}
	;

dimension : T2D
	{
		$$ = newnode(T2D, NULL, NULL);
	}
	| T3D
	{
		$$ = newnode(T3D, NULL, NULL);
	}
	| T2DONLY
	{
		$$ = newnode(T2DONLY, NULL, NULL);
	}
	| T3DONLY
	{
		$$ = newnode(T3DONLY, NULL, NULL);
	}
	;

outputformat : TPAINTJET
        {
                $$ = newnode(TPAINTJET, NULL, NULL);
        }
        | TTEXSPEC
        {
                $$ = newnode(TTEXSPEC, NULL, NULL);
        }
        | TLASERJET
        {
                $$ = newnode(TLASERJET, NULL, NULL);
        }
        ;


numlist : point numlist
	{

	     $1->sibling = $2;
	}
	| point 
	;

point   : num num
	{
	    $$ = newnode(TPOINT, $1, $2);
	}
	;

numlist3d : point3d numlist3d
	{
		$1->sibling = $2;
	}
	|  point3d
	;

point3d : num num num
	{
		XGP_node   *p1;

		p1= newnode(TPOINT3D, $1, $2);
		$$ = newnode(TPOINT3D, p1, $3);
	}
	;

string  : TSCONST
	{
		$$ = newnode(TSCONST, NULL, NULL);
		$$->sval = $1;
	}
num	: fnum
	| inum
	;

fnum	: TFCONST
	{
		$$ = newnode (TFCONST, NULL, NULL);
		$$->fval = $1;
	}
	;

inum	: TICONST
	{
		$$ = newnode (TICONST, NULL, NULL);
		$$->ival = $1;
	}
	;
%%

XGP_node* makesibs(p1,p2)
	XGP_node  *p1, *p2;

	{
	  if (p1->sibling == NULL) {
	    p1->sibling = p2;
	    return(p1);
	} else {
	    (void) makesibs(p1->sibling,p2); 
	    return(p1);
	}
	}

/* Dan Bennett  10/11/89   */

XGP_node* siblings(ntype1,ntype2,p1,p2)
	 int      ntype1, ntype2;
	 XGP_node *p1,*p2;
	 {
	     XGP_node *t1,*t2;

	     t1 = newnode(ntype1, p1, p2);
             t2 = newnode(ntype2, p1, p2);
	     t2->sibling = t1;
	     return(t2);
	 }

XGP_node *newnode (ntype, p1, p2)
int		ntype;
XGP_node	*p1, *p2;
{
	XGP_node	*ptr;
	
	ptr = (XGP_node *)malloc (sizeof (XGP_node));
	bzero ((char *)ptr, sizeof (XGP_node));
	ptr->node_type = ntype;
	ptr->child[0] = p1;
	ptr->child[1] = p2;
	ptr->sibling = NULL;
	return (ptr);
}

yyerror (s)
char	*s;
{
	dberror(s);
}

/*
 * resynchronize after an error - skip everything till all the open
 * parans have been closed and start again.
 */
resync ()
{
	while (xgp_paran_ct > 0) { 
		/* then that many r-parans are forthcoming */
		if (yychar = yylex (), yychar == 0)
			/* EOF met */
			break;
	}
	yychar = -1;
	return;
}
