/********************************************************************/
/* The shell for this file and for all of the parser scanner was     */
/* was provided by Sanjiva Weerawarana and was origionally developed */
/* for GENCRAY  his Masters Thesis                                   */
/*********************************************************************/

/*
#define	STDEBUG
*/

#include	<stdio.h>
#include	<ctype.h>
#ifdef SYSV
#	include	<string.h>
#	define index   strchr
#	define rindex  strrchr
#else
#	include	<strings.h>
#endif

#include        "GraphP.h"

#include	"ObjectsP.h"
#include	"pars.h"
#include	"y.tab.h"

#ifndef TRUE
#define	TRUE	1
#endif
#ifndef FALSE
#define	FALSE	0
#endif

/* temporary structure definitions to help build the keywords table */
struct {
	char	*str;
	int	tokencode;
} keyword[] = {	
		{ "DrawPrimitive",	TDRAWPRIM },
		{ "drawprimitive",	TDRAWPRIM },
		{ "Combine",		TCOMBINE },
		{ "Move",		TMOVE },
		{ "Copy",		TCOPY },
		{ "Set",		TSET },
		{ "Change",		TCHANGE },
		{ "Remove",		TREMOVE },
		{ "Comment",            TCOMMENT },
		{ "Reset",              TRESET },
		{ "Line",		TLINE },
		{ "Line3D",		TLINE3D },
		{ "Point",		TPOINT },
		{ "Point3D",		TPOINT3D },
		{ "Circle",             TCIRCLE },
		{ "Circle3D",           TCIRCLE },
		{ "Cone",		TCONE },
		{ "Cylinder",		TCYLENDER  },
		{ "Sphere",             TSPHERE },
		{ "Ellipse",            TELIPSE },
		{ "Polygon",            TPOLYGON },
		{ "Polygon3D",          TPOLYGON3D },
		{ "surface",            TSURFACE },
		{ "Surface",            TSURFACE },
		{ "polyline", 		TPOLYLINE },
		{ "Polyline", 		TPOLYLINE },
		{ "Polyline3D",         TPOLYLINE3D },
		{ "Text",		TTEXT },
		{ "Text3D",		TTEXT },
		{ "Font",		TFONT },
		{ "LoadFont",		TLOADFONT },
		{ "Regular",		TREGULAR },
		{ "None",		TNONE },
		{ "Default",   		TDEFAULT },
		{ "Bold",		TBOLD },
		{ "Italic",		TITALIC },
		{ "Greek",		TGREEK },
		{ "RegularPolygon",     TREGPOLY },
		{ "Visible",		TVISIBLE },
		{ "Invisible",		TINVISIBLE },
		{ "Up",			TUP },
		{ "Down",		TDOWN },
		{ "Center",		TCENTER },
		{ "Right",		TRIGHT },
		{ "Left",		TLEFT },
		{ "True",    		TTRUE },
		{ "False",		TFALSE },
		{ "Viewport",           TVIEWPORT },
		{ "MinMax",             TMINMAX },
		{ "EyePosition",        TEYEPOS },
		{ "FocusPosition",      TFOCPOS },
		{ "Centered",		TCENTERED },
		{ "DistanceFromEye",    TDISTFROMI },
		{ "ViewportSize",       TVIEWSIZE },
		{ "ViewportRotation",   TVIEWROT },
                { "Rotate",             TROTATE },
		{ "Translate",          TTRANSLATE },
		{ "Scale",		TSCALE },
		{ "Transformation",     TTRANSFORM },
		{ "Replace",            TREPLACE },
		{ "Append",		TAPPEND },
		{ "Identity",		TIDENTITY },
		{ "TwoD",		T2D },
		{ "ThreeD",   		T3D },
		{ "TwoDOnly",           T2DONLY },
		{ "ThreeDOnly",         T3DONLY },
		{ "Hidden",             THIDDEN },
		{ "LineStyle",          TLINEST },
		{ "LineWidth",          TLINEWID },
		{ "Color",              TCOLOR },
		{ "Solid",		TSOLID },
		{ "Dashed",		TDASHED },
		{ "Double",		TDOUBLE },
		{ "Quit",		TQUIT},
		{ "Sleep",    		TSLEEP},
		{ "Execute",		TEXECUTE},
		{ "Save", 	        TSAVE},
		{ "SaveCommands",       TSAVECOMM},
		{ "SaveCommandsStop",   TSAVECOMMSTOP},
		{ "Alias",              TALIAS},
		{ "Visibility",         TVISIBILITY },
		{ "AutoAdjust",		TAUTOADJ },
		{ "RLineWidth",		TWIDTH },
		{ "RLineWidthX",        TXWIDTH },
		{ "RLineWidthY",        TYWIDTH },
		{ "RLineStyle",         TRSTYLE },
		{ "RLineStyleX",        TRXSTYLE },
		{ "RLineStyleY",        TRYSTYLE },
		{ "Center",		TCENTER },
		{ "LabelVisibility",    TLABVIS },
		{ "LabelXVisibility",   TLABXVIS },
		{ "LabelYVisibility",   TLABYVIS },
		{ "LabelFont",		TLABFONT },
		{ "LabelXFont",         TLABXFONT },
		{ "LabelYFont",         TLABYFONT },
		{ "LabelPrecision",	TLABPRE },
		{ "LabelXPrecision",    TLABXPRE },
		{ "LabelYPrecision",    TLABYPRE },
		{ "LabelSkip",		TLABSKIP },
		{ "LabelXSkip",         TLABXSKIP },
		{ "LabelYSkip",         TLABYSKIP },
		{ "LabelJustify", 	TLABJUST },
		{ "LabelXJustify",      TLABXJUST },
		{ "LabelYJustify",      TLABYJUST },
		{ "LabelColor",         TLABCOLOR },
		{ "LabelRange",		TLABRANGE },
		{ "LabelXRange",        TLABXRANGE },
		{ "LabelYRange",        TLABYRANGE },
		{ "TickmarkVisibility", TTICKVIS },
		{ "TickmarkXVisibility",TTICKXVIS },
		{ "TickmarkYVisibility",TTICKYVIS },
		{ "TickmarkScale",	TTICKSCALE },
		{ "TickmarkXScale",     TTICKXSCALE },
		{ "TickmarkYScale",     TTICKYSCALE },
		{ "TickmarkRange",	TTICKRANGE },
		{ "TickmarkXRange",     TTICKXRANGE },
		{ "TickmarkYRange",     TTICKYRANGE },
		{ "TickmarkColor",      TTICCOLOR },
		{ "Xaxis",		TXAXIS },
		{ "Yaxis",		TYAXIS },
		{ "Zaxis",              TZAXIS },
		{ "VolumeBoxDimension",   TVIEWDIM },
		{ "VolumeBoxCoordinates", TVIEWCOR },
		{ "OutputFormat",       TOUTFORM },
		{ "OutputFile",         TOUTFILE },
		{ "SizeMeasure",        TSIZEMES },
		{ "DotsPerUnit",        TDPI },
		{ "PictureLength",      TPICTLEN },
		{ "PictureWidth",       TPICTWID },
		{ "Inches",             TINCH },
		{ "Centimeters",        TCENT },
		{ "PaintJet",           TPAINTJET },
		{ "LaserJet",           TLASERJET },
		{ "TexSpecial",         TTEXSPEC },
		{ "HardCopy",           THARDCOPY },
		{ "Output",             TOUTPUT },
		{ "Colormap",           TCOLORMAP },
		{ "Private",            TPRIVATE },
		{ "Public",             TPUBLIC },
		{ "SUImode",            TSUIMODE },
		{ "HiddenLineMode",     THLHS },
		{ "Redraw",		TREDRAW },
		{ "Dimension",          TDIMENSION },
		{ "On",                 TON },
		{ "Off",                TOFF },
		{ "Inquire",            TINQUIRE },
		{ "Object",             TOBJECT },
		{ "", -1 }
};

char		*XGPmalloc();
/* htblentry *hashtbl[101];*/

extern GraphWidget widgetholdforparse;

/* returns a 0-based index to hashtable, depending on str */
int hash (str)
char	*str;
{
	register short	i = 0;
	register char	c;
	register int	result = 0;
	
#ifdef STDEBUG
fprintf (stderr, "XGP: Entered stbl.c:hash()\n");
#endif
	for (; i < strlen (str); i++)
		result = result*16 + str[i];
	result %= 101;		/* 101 = number of entries in hash table */
	while (result < 0)
		result += 101;
	return (result);
}

/* This function installs/looks up lexemes in the symbol table.
lexeme is the actual string
tokencode is either what the lexeme's tokencode will be (if it's not already
	in the symbol table or its useless)
*/

stblentry *stblget (lexeme, tokencode)
char	*lexeme;
int	tokencode;
{
	htblentry *hp;
	int		hindex;			/* index rtned by hash() */
        int             foobaz;

	foobaz = 10;

#ifdef STDEBUG
fprintf (stderr, "XGP: Entered stbl.c:stblget(%s,%d)\n", lexeme, tokencode);
#endif
if (hp = widgetholdforparse->graph.hashtbl[hindex = hash(lexeme)], hp == NULL)
	{
		/* first is null, no need to search */
		hp =widgetholdforparse->graph.hashtbl[hindex] = (htblentry *)XGPmalloc (widgetholdforparse,sizeof (htblentry));
	}
	else 
	{
		while (1)
		{
			if (!strcmp (hp->sp->name, lexeme))	/* found! */
				return (hp->sp);
			if (hp->next != NULL)
				hp = hp->next;
			else
				break;
		}
		hp->next = (htblentry *)XGPmalloc (widgetholdforparse,
		     sizeof (htblentry));
		hp = hp->next;
	}
	hp->next = NULL;
	hp->sp = (stblentry *)XGPmalloc (widgetholdforparse,
	     sizeof (stblentry));
	hp->sp->length = strlen (lexeme);
	if (hp->sp->length >= foobaz) {
	   foobaz = hp->sp->length+1;
	}
	hp->sp->name = (char *)XGPmalloc (widgetholdforparse,
	     foobaz*sizeof(char));
	strcpy (hp->sp->name, lexeme);
	hp->sp->tokencode = tokencode;
	return (hp->sp);
}

stblentry *lookup (lexeme)
char	*lexeme;
{
	htblentry *hp;
	
#ifdef STDEBUG
fprintf (stderr, "XGP: Entered stbl.c:lookup()\n");
#endif
	for (hp = widgetholdforparse->graph.hashtbl[hash(lexeme)]; hp != NULL; hp = hp->next)
		if (!strcmp (hp->sp->name, lexeme))	/* found */
			return (hp->sp);
	return (NULL);
}

/* preload all the keywords into the hashtable */
preload ()
{
	register int	i;

#ifdef STDEBUG
fprintf (stderr, "XGP: Entered stbl.c:preload()\n");
#endif
	/* initialize all the ptrs in hashtbl to null */
	for (i = 0; i < 101; i++)
		widgetholdforparse->graph.hashtbl[i] = NULL;
	for (i = 0; keyword[i].tokencode != -1; i++)
		stblget (keyword[i].str, keyword[i].tokencode);
}
